'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var accesscontrol_1 = require("accesscontrol");
var _ = require('lodash');
var CrudAccessControl = /** @class */ (function () {
    function CrudAccessControl(resource, grants) {
        this.resource = resource;
        this.grants = grants;
        this.ac = new accesscontrol_1.AccessControl(grants);
    }
    CrudAccessControl.prototype.check = function (user, op) {
        var _ac = this.ac;
        //check for `any` clearence: the user has accesss to any document in the target collection
        var grant;
        logger_1.default.trace("check can " + user.roles + " " + op + "Any for " + this.resource);
        var anyPermission = this.doCheck(_ac, user.roles, op, 'Any', this.resource);
        if (anyPermission.granted) {
            grant = new Grant(anyPermission, 'any');
            grant.addVerifiedRoles(user.roles);
        }
        else {
            //check for `own` clearence:
            var ownGrant = new Grant();
            ///check for `own` clearence for 'user'
            logger_1.default.trace("check can " + user.roles + " " + op + "Own for " + this.resource);
            var userOwnPermission = this.doCheck(_ac, user.roles, op, 'Own', this.resource);
            ownGrant.addVerifiedRoles(user.roles);
            //check for `organization:own` clearence
            if (userOwnPermission.granted) {
                ownGrant.addGrant(new Grant(userOwnPermission, 'own', 'user'));
            }
            //check for 'own' clearence for 'organization'
            var orgRoles = this.getOrgRoles(user);
            logger_1.default.trace("check can " + orgRoles + " " + op + "Own for " + this.resource);
            var organizationOwnPermission = this.doCheck(_ac, orgRoles, op, 'Own', this.resource);
            if (organizationOwnPermission.granted) {
                ownGrant.addGrant(new Grant(organizationOwnPermission, 'own', 'organization'));
            }
            if (ownGrant.granted) {
                grant = ownGrant;
            }
            else {
                grant = new Grant(); /* granted false */
            }
        }
        return grant;
    };
    CrudAccessControl.prototype.doCheck = function (_ac, roles, op, type, resource) {
        var permission;
        try {
            permission = _ac.can(roles)[op + type](resource);
        }
        catch (err) {
            logger_1.default.warn(err.message);
            permission = { granted: false };
        }
        return permission;
    };
    ;
    CrudAccessControl.prototype.getOrgRoles = function (user) {
        var roles = [];
        if (user.organization && user.organization.ref
            && user.organization.ref.code) {
            var org = user.organization;
            roles.push(org.ref.code + ":" + org.role);
        }
        return roles;
    };
    return CrudAccessControl;
}());
exports.CrudAccessControl = CrudAccessControl;
var Grant = /** @class */ (function () {
    function Grant(permission, _type, _ownerType) {
        this.granted = false;
        this.type = 'none'; /* none, any, own */
        this.ownerTypes = []; /* empty, organziaiton, user*/
        this.permissions = [];
        this.verifiedRoles = [];
        if (_type) {
            this.type = _type;
        }
        this.addPermission(permission);
        this.addOwnerType(_ownerType);
    }
    Grant.prototype.addGrant = function (grant) {
        if (!grant) {
            return;
        }
        this.type = grant.type;
        this.addPermission(grant.permissions[0]);
        this.addOwnerType(grant.ownerTypes[0]);
    };
    Grant.prototype.addOwnerType = function (ownerType) {
        if (!ownerType) {
            return;
        }
        this.ownerTypes.push(ownerType);
    };
    Grant.prototype.addPermission = function (permission) {
        if (!permission || !permission.granted) {
            return;
        }
        this.permissions.push(permission);
        this.granted = this.granted || permission.granted;
    };
    Grant.prototype.addVerifiedRoles = function (roles) {
        if (!roles) {
            return;
        }
        this.verifiedRoles = this.verifiedRoles.concat(roles);
    };
    return Grant;
}());
exports.Grant = Grant;
