"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var createError = require('http-errors');
var promise_grants_utils_1 = require("../utils/promise-grants.utils");
function respondWithResult(res, statusCode) {
    statusCode = statusCode || 200;
    return function (entity) {
        if (entity) {
            res.status(statusCode).json(entity);
        }
    };
}
exports.respondWithResult = respondWithResult;
function handleEntityNotFound(res) {
    return function (entity) {
        if (!entity) {
            throw createError(404);
        }
        return entity;
    };
}
exports.handleEntityNotFound = handleEntityNotFound;
function handleError(res, statusCode) {
    statusCode = statusCode || 500;
    return function (err) {
        logger_1.default.debug('responding with http error');
        logger_1.default.debug(err);
        if (err.statusCode) {
            res.status(err.statusCode).send(err.message);
            return;
        }
        if (err.name === 'ValidationError')
            statusCode = 400;
        res.status(statusCode).send(err);
    };
}
exports.handleError = handleError;
function successMessageResult() {
    return function () {
        return 'Success';
    };
}
exports.successMessageResult = successMessageResult;
function baseHandle(req, res, promisedAc, op, handleFnc) {
    var self = this;
    promisedAc
        .then(function (accessControl) {
        var permission = accessControl.check(req.user, op);
        return promise_grants_utils_1.assertGranted(permission);
    })
        .then(function (permission) {
        return handleFnc(permission, req.user);
    })
        .then(self.respondWithResult(res))
        .catch(self.handleError(res));
}
exports.baseHandle = baseHandle;
